// -*- c++ -*-
/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#if !defined __CLIENTCONNECTION_H
#define __CLIENTCONNECTION_H

#include <string>
#include "dvbd.h"
#include "unixclientsocket.h"

class UnixClientSocket;

/** Establish a pair of connections with the server */
class ClientConnection {
public:  
  ClientConnection( const std::string &socketFile );

  operator bool () const;

  bool getChannels(const std::string &type, int priority, StringList &channels);

  std::string getError() const;

  UnixClientSocket& data();
  UnixClientSocket& control();

  void checkSchedule(StringList&);

  int schedule(const std::string &type, const std::string &channel,
	       bool convertToPS, time_t when, unsigned duration, int priority,
	       const std::string &outputFile, const std::string &extra);

  bool removeJob(int job);

  bool listSchedule(StringList &);

  bool tune(const std::string &type, const std::string &channel, bool convertToPS, int priority);

  bool getNextJobTime(time_t &nextJobTime);

  bool listConnections(StringList &);

private:
  UnixClientSocket control_, data_;
  bool good_;
  std::string error_;
};

#endif // __CLIENTCONNECTION_H
