// -*- c++ -*-
/*
  Copyright 2003 John Knottenbelt
  
  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.
 
  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
 
  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
*/

#if !defined __SOURCE_H
#define __SOURCE_H

#include <list>

// Priority base class

class Priority {
public:
  virtual ~Priority();
  virtual int getPriority() const = 0;
  virtual void setPriority( int newPriority );
};

class SavePriority {
public:
  SavePriority(Priority *p, int newPriority)
    : p(p), savePriority(p->getPriority())
  {
    p->setPriority(newPriority);
  }

  ~SavePriority()
  {
    p->setPriority(savePriority);
  }

private:
  Priority *p;
  int savePriority;
};

// Sink abstract base class

class Source;

class Sink : public virtual Priority {
 public:
  virtual ~Sink();
  virtual void receiveData(unsigned char *data, unsigned size) = 0;
  virtual void notifyUnsubscribe(Source *source);
};

typedef std::list<Source *> SourceList;
typedef std::list<Sink *> SinkList;

// Source abstract base class
class Source : public virtual Priority {
public:
  virtual ~Source();

  virtual bool subscribe(Sink *s);
  virtual void unsubscribe(Sink *s);

  void unsubscribeAll();
  int unsubscribe( int priority );

  int getNumSubscribers() const { return subscribers.size(); };
  int getNumSubscribers(int priority) const;
  virtual int getPriority() const;

protected:
  virtual void sendData(unsigned char *data, unsigned size) const;

  SinkList subscribers;
};


#endif // __SOURCE_H
